@@ -17,6 +17,7 @@
 #include "stat_cache.h"
 
 #include "plugin.h"
+#include "plugins.h"
 
 #include "sock_addr_cache.h"
 
@@ -157,7 +158,6 @@ static void connection_handle_shutdown(connection *con) {
 	plugins_call_handle_connection_shut_wr(con);
 
 	connection_reset(con);
-	++con->srv->con_closed;
 
 	/* close the connection */
 	if (con->fd >= 0
@@ -189,8 +189,6 @@ static void connection_handle_response_end_state(request_st * const r, connectio
 	/* (even if error, connection dropped, as long as http_status is set) */
 	if (r->http_status) plugins_call_handle_request_done(r);
 
-	if (r->state != CON_STATE_ERROR) ++con->srv->con_written;
-
 	if (r->reqbody_length != r->reqbody_queue.bytes_in
 	    || r->state == CON_STATE_ERROR) {
 		/* request body may not have been read completely */
@@ -206,8 +204,9 @@ static void connection_handle_response_end_state(request_st * const r, connectio
 		request_reset(r);
 		con->is_readable = 1; /* potentially trigger optimistic read */
 		/*(accounting used by mod_accesslog for HTTP/1.0 and HTTP/1.1)*/
-		r->bytes_read_ckpt = con->bytes_read;
-		r->bytes_written_ckpt = con->bytes_written;
+		/*(overloaded to detect next bytes recv'd on keep-alive con)*/
+		r->bytes_read_ckpt = r->read_queue.bytes_in;
+		r->bytes_written_ckpt = r->write_queue.bytes_out;
 #if 0
 		r->start_hp.tv_sec = log_epoch_secs;
 		con->read_idle_ts = log_monotonic_secs;
@@ -286,7 +285,9 @@ connection_write_chunkqueue (connection * const con, chunkqueue * const restrict
         /* Linux: put a cork into socket as we want to combine write() calls
          * but only if we really have multiple chunks including non-MEM_CHUNK
          * (or if multiple chunks and TLS), and only if TCP socket */
-        if (NULL != c || (max_bytes > 16384 && con->is_ssl_sock)) {
+        /* (max_bytes may have been reduced by connection_write_throttle(),
+         *  but not bothering to check; might result in some extra corking) */
+        if (NULL != c || (con->is_ssl_sock && chunkqueue_length(cq) > 16384)) {
             const int sa_family = sock_addr_get_family(&con->srv_socket->addr);
             if (sa_family == AF_INET || sa_family == AF_INET6) {
                 corked = 1;
@@ -311,7 +312,6 @@ connection_write_chunkqueue (connection * const con, chunkqueue * const restrict
   #endif
 
     written = cq->bytes_out - written;
-    con->bytes_written += written;
     con->bytes_written_cur_second += written;
     request_st * const r = &con->request;
     if (r->conf.global_bytes_per_second_cnt_ptr)
@@ -333,7 +333,6 @@ connection_write_1xx_info (request_st * const r, connection * const con)
     int rc = con->network_write(con, cq, MAX_WRITE_LIMIT);
 
     written = cq->bytes_out - written;
-    con->bytes_written += written;
     con->bytes_written_cur_second += written;
     if (r->conf.global_bytes_per_second_cnt_ptr)
         *(r->conf.global_bytes_per_second_cnt_ptr) += written;
@@ -350,6 +349,9 @@ connection_write_1xx_info (request_st * const r, connection * const con)
              * Note: sending of remainder of 1xx might be delayed
              * until next set of response headers are sent */
             con->write_queue = chunkqueue_init(NULL);
+            /* (copy bytes for accounting purposes in event of failure) */
+            con->write_queue->bytes_in = cq->bytes_out; /*(yes, bytes_out)*/
+            con->write_queue->bytes_out = cq->bytes_out;
             chunkqueue_append_chunkqueue(con->write_queue, cq);
         }
     }
@@ -556,10 +558,7 @@ static void connection_reset(connection *con) {
 	r->bytes_read_ckpt = 0;
 	r->bytes_written_ckpt = 0;
 	con->is_readable = 1;
-
-	con->bytes_written = 0;
 	con->bytes_written_cur_second = 0;
-	con->bytes_read = 0;
 }
 
 
@@ -653,7 +652,7 @@ static int connection_handle_read_state(connection * const con)  {
 
     if (con->request_count > 1) {
         discard_blank = 1;
-        if (con->bytes_read == r->bytes_read_ckpt) {
+        if (cq->bytes_in == r->bytes_read_ckpt) {
             keepalive_request_start = 1;
             if (NULL != c) { /* !chunkqueue_is_empty(cq)) */
                 pipelined_request_start = 1;
@@ -726,7 +725,7 @@ static int connection_handle_read_state(connection * const con)  {
     } while ((c = connection_read_header_more(con, cq, c, clen)));
 
     if (keepalive_request_start) {
-        if (con->bytes_read > r->bytes_read_ckpt) {
+        if (cq->bytes_in > r->bytes_read_ckpt) {
             /* update r->start_hp.tv_sec timestamp when first byte of
              * next request is received on a keep-alive connection */
             r->start_hp.tv_sec = log_epoch_secs;
@@ -748,6 +747,9 @@ static int connection_handle_read_state(connection * const con)  {
     char * const hdrs = c->mem->ptr + hoff[1];
 
     if (con->request_count > 1) {
+        /* adjust r->bytes_read_ckpt for http_request_stats_bytes_in()
+         * (headers_len is still in cq; marked written, bytes_out incr below) */
+        r->bytes_read_ckpt = cq->bytes_out;
         /* clear buffers which may have been kept for reporting on keep-alive,
          * (e.g. mod_status) */
         request_reset_ex(r);
@@ -778,6 +780,7 @@ static int connection_handle_read_state(connection * const con)  {
         /*(Upgrade: h2c over cleartext does not have SNI; no COMP_HTTP_HOST)*/
         r->conditional_is_valid = (1 << COMP_SERVER_SOCKET)
                                 | (1 << COMP_HTTP_REMOTE_IP);
+        r->bytes_read_ckpt = 0;
         /*connection_handle_write(r, con);*//* defer write to network */
         return 0;
     }
@@ -878,18 +881,9 @@ static int connection_read_cq(connection *con, chunkqueue *cq, off_t max_bytes)
         if (len != (ssize_t)mem_len) {
             /* we got less then expected, wait for the next fd-event */
             con->is_readable = 0;
-
-            if (len > 0) {
-                con->bytes_read += len;
-                return 0;
-            }
-            else if (0 == len) /* other end close connection -> KEEP-ALIVE */
-                return -2;     /* (pipelining) */
-            else
-                return connection_read_cq_err(con);
+            return len > 0 ? 0 : 0 == len ? -2 : connection_read_cq_err(con);
         }
 
-        con->bytes_read += len;
         max_bytes -= len;
 
         int frd;
@@ -918,7 +912,6 @@ connection *connection_accepted(server *srv, const server_socket *srv_socket, so
 #if 0
 		log_error(srv->errh, __FILE__, __LINE__, "accepted() %d", cnt);
 #endif
-		srv->con_opened++;
 
 		con = connections_get_new_connection(srv);
 
@@ -955,27 +948,19 @@ connection *connection_accepted(server *srv, const server_socket *srv_socket, so
 
 __attribute_cold__
 __attribute_noinline__
-static const char *
-connection_get_state (request_state_t state)
+__attribute_nonnull__()
+static void
+connection_log_state (const request_st * const r, const char * const tag)
 {
-    switch (state) {
-      case CON_STATE_CONNECT:        return "connect";
-      case CON_STATE_READ:           return "read";
-      case CON_STATE_READ_POST:      return "readpost";
-      case CON_STATE_WRITE:          return "write";
-      case CON_STATE_CLOSE:          return "close";
-      case CON_STATE_ERROR:          return "error";
-      case CON_STATE_HANDLE_REQUEST: return "handle-req";
-      case CON_STATE_REQUEST_START:  return "req-start";
-      case CON_STATE_REQUEST_END:    return "req-end";
-      case CON_STATE_RESPONSE_START: return "resp-start";
-      case CON_STATE_RESPONSE_END:   return "resp-end";
-      default:                       return "(unknown)";
-    }
+    buffer * const tb = r->tmp_buf;
+    buffer_clear(tb);
+    http_request_state_append(tb, r->state);
+    log_error(r->conf.errh, __FILE__, __LINE__,
+      "fd:%d id:%d state:%s%s", r->con->fd, r->h2id, tb->ptr, tag);
 }
 
 
-static void connection_state_machine_h2 (request_st *h2r, connection *con);
+static void connection_state_machine_h2 (connection * const con);
 
 
 static void
@@ -983,11 +968,8 @@ connection_state_machine_loop (request_st * const r, connection * const con)
 {
 	request_state_t ostate;
 	do {
-		if (r->conf.log_state_handling) {
-			log_error(r->conf.errh, __FILE__, __LINE__,
-			  "state for fd:%d id:%d %s", con->fd, r->h2id,
-			  connection_get_state(r->state));
-		}
+		if (r->conf.log_state_handling)
+			connection_log_state(r, "");
 
 		switch ((ostate = r->state)) {
 		case CON_STATE_REQUEST_START: /* transient */
@@ -1007,7 +989,7 @@ connection_state_machine_loop (request_st * const r, connection * const con)
 			if (!connection_handle_read_state(con)) {
 				if (r->http_version == HTTP_VERSION_2) {
 					connection_transition_h2(r, con);
-					connection_state_machine_h2(r, con);
+					connection_state_machine_h2(con);
 					return;
 				}
 				break;
@@ -1060,9 +1042,8 @@ connection_state_machine_loop (request_st * const r, connection * const con)
 			break;
 		case CON_STATE_CONNECT:
 			break;
-		default:
-			log_error(r->conf.errh, __FILE__, __LINE__,
-			  "unknown state: %d %d", con->fd, r->state);
+		default:/*(should not happen)*/
+			/*connection_log_state(r, "");*/ /*(unknown state)*/
 			break;
 		}
 	} while (ostate != (request_state_t)r->state);
@@ -1195,6 +1176,7 @@ connection_request_end_h2 (request_st * const h2r, connection * const con)
     if (h2r->keep_alive >= 0) {
         h2r->keep_alive = -1;
         h2_send_goaway(con, H2_E_NO_ERROR);
+        http_response_delay(con);
     }
     else /*(abort connection upon second request to close h2 connection)*/
         h2_send_goaway(con, H2_E_ENHANCE_YOUR_CALM);
@@ -1202,9 +1184,10 @@ connection_request_end_h2 (request_st * const h2r, connection * const con)
 
 
 static void
-connection_state_machine_h2 (request_st * const h2r, connection * const con)
+connection_state_machine_h2 (connection * const con)
 {
     h2con * const h2c = con->h2;
+    request_st * const h2r = &con->request;
 
     if (h2c->sent_goaway <= 0
         && (chunkqueue_is_empty(con->read_queue) || h2_parse_frames(con))
@@ -1242,27 +1225,16 @@ connection_state_machine_h2 (request_st * const h2r, connection * const con)
          * con->write_queue, consider setting limit on how much is staged
          * for sending on con->write_queue: adjusting max_bytes down */
 
-        /* XXX: TODO: process requests in stream priority order */
         for (uint32_t i = 0; i < h2c->rused; ++i) {
             request_st * const r = h2c->r[i];
             /* future: might track read/write interest per request
              * to avoid iterating through all active requests */
 
-          #if 0
-            const int log_state_handling = r->conf.log_state_handling;
-            if (log_state_handling)
-                log_error(r->conf.errh, __FILE__, __LINE__,
-                  "state at enter %d %d %s", con->fd, r->h2id,
-                  connection_get_state(r->state));
-          #endif
-
             connection_state_machine_loop(r, con);
 
           #if 0
-            if (log_state_handling)
-                log_error(r->conf.errh, __FILE__, __LINE__,
-                  "state at exit %d %d %s", con->fd, r->h2id,
-                  connection_get_state(r->state));
+            if (r->conf.log_state_handling)
+                connection_log_state(r, " at loop exit");
           #endif
 
             if (r->state < CON_STATE_WRITE)
@@ -1275,8 +1247,12 @@ connection_state_machine_h2 (request_st * const h2r, connection * const con)
                         || (r->conf.stream_response_body
                             & (FDEVENT_STREAM_RESPONSE
                               |FDEVENT_STREAM_RESPONSE_BUFMIN)))) {
-                    uint32_t dlen =
-                      max_bytes > 32768 ? 32768 : (uint32_t)max_bytes;
+                    /*(subtract 9 byte HTTP/2 frame overhead from each 16k DATA
+                     * frame for more efficient sending of large files)*/
+                    /*(use smaller max per stream if marked 'incremental' (w/ 0)
+                     * to give more streams a chance to send in parallel)*/
+                    uint32_t dlen = (r->h2_prio & 1) ? 32768-18 : 8192;
+                    if (dlen > (uint32_t)max_bytes) dlen = (uint32_t)max_bytes;
                     dlen = h2_send_cqdata(r, con, &r->write_queue, dlen);
                     if (dlen) { /*(do not resched (spin) if swin empty window)*/
                         max_bytes -= (off_t)dlen;
@@ -1290,7 +1266,7 @@ connection_state_machine_h2 (request_st * const h2r, connection * const con)
 
                 connection_set_state(r, CON_STATE_RESPONSE_END);
                 if (__builtin_expect( (r->conf.log_state_handling), 0))
-                    connection_state_machine_loop(r, con);
+                    connection_log_state(r, "");
             }
 
             {/*(r->state==CON_STATE_RESPONSE_END || r->state==CON_STATE_ERROR)*/
@@ -1361,20 +1337,13 @@ connection_state_machine_h2 (request_st * const h2r, connection * const con)
 
 
 static void
-connection_state_machine_h1 (request_st * const r, connection * const con)
+connection_state_machine_h1 (connection * const con)
 {
-	const int log_state_handling = r->conf.log_state_handling;
-	if (log_state_handling) {
-		log_error(r->conf.errh, __FILE__, __LINE__,
-		  "state at enter %d %s", con->fd, connection_get_state(r->state));
-	}
-
+	request_st * const r = &con->request;
 	connection_state_machine_loop(r, con);
 
-	if (log_state_handling) {
-		log_error(r->conf.errh, __FILE__, __LINE__,
-		  "state at exit: %d %s", con->fd, connection_get_state(r->state));
-	}
+	if (r->conf.log_state_handling)
+		connection_log_state(r, " at loop exit");
 
 	connection_set_fdevent_interest(r, con);
 }
@@ -1383,11 +1352,10 @@ connection_state_machine_h1 (request_st * const r, connection * const con)
 void
 connection_state_machine (connection * const con)
 {
-    request_st * const r = &con->request;
-    if (r->http_version == HTTP_VERSION_2)
-        connection_state_machine_h2(r, con);
+    if (con->h2)
+        connection_state_machine_h2(con);
     else /* if (r->http_version <= HTTP_VERSION_1_1) */
-        connection_state_machine_h1(r, con);
+        connection_state_machine_h1(con);
 }
 
 
@@ -1529,7 +1497,8 @@ static void connection_check_timeout (connection * const con, const unix_time64_
                   "increase server.max-write-idle",
                   con->dst_addr_buf.ptr,
                   BUFFER_INTLEN_PTR(&r->target),
-                  (long long)con->bytes_written, (int)r->conf.max_write_idle);
+                  (long long)con->write_queue->bytes_out,
+                  (int)r->conf.max_write_idle);
             }
             connection_set_state_error(r, CON_STATE_ERROR);
             changed = 1;
@@ -1543,8 +1512,9 @@ static void connection_check_timeout (connection * const con, const unix_time64_
     if (0 == (t_diff = cur_ts - con->connection_start)) t_diff = 1;
 
     if (con->traffic_limit_reached &&
-        (r->conf.bytes_per_second == 0 ||
-         con->bytes_written < (off_t)r->conf.bytes_per_second * t_diff)) {
+        (r->conf.bytes_per_second == 0
+         || con->write_queue->bytes_out
+              < (off_t)r->conf.bytes_per_second * t_diff)) {
         /* enable connection again */
         con->traffic_limit_reached = 0;
 
