@@ -1,5 +1,6 @@
 #include "first.h"
 
+#include "plugins.h"
 #include "plugin.h"
 #include "base.h"
 #include "array.h"
@@ -8,8 +9,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-array plugin_stats; /* global */
-
 #ifdef HAVE_VALGRIND_VALGRIND_H
 # include <valgrind/valgrind.h>
 #endif
@@ -165,8 +164,8 @@ int plugins_load(server *srv) {
 
 	for (uint32_t i = 0; i < srv->srvconf.modules->used; ++i) {
 		const buffer * const module = &((data_string *)srv->srvconf.modules->data[i])->value;
-		buffer_copy_path_len2(tb, BUF_PTR_LEN(srv->srvconf.modules_dir),
-		                          BUF_PTR_LEN(module));
+		buffer_copy_string(tb, srv->srvconf.modules_dir);
+		buffer_append_path_len(tb, BUF_PTR_LEN(module));
 #if defined(__WIN32) || defined(__CYGWIN__)
 		buffer_append_string_len(tb, CONST_STR_LEN(".dll"));
 #else
